@@ -17,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -118,7 +118,13 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 #define PushSegmentStack(up,left,right,delta) \
 { \
   if (s >= (segment_stack+MaxStacksize)) \
-    ThrowBinaryException(DrawError,"SegmentStackOverflow",image->filename) \
+    { \
+      segment_info=RelinquishVirtualMemory(segment_info); \
+      image_view=DestroyCacheView(image_view); \
+      floodplane_view=DestroyCacheView(floodplane_view); \
+      floodplane_image=DestroyImage(floodplane_image); \
+      ThrowBinaryException(DrawError,"SegmentStackOverflow",image->filename) \
+    } \
   else \
     { \
       if ((((up)+(delta)) >= 0) && (((up)+(delta)) < (ssize_t) image->rows)) \
@@ -147,10 +153,9 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     *segment_info;
 
   PixelInfo
-    fill_color,
     pixel;
 
-  register SegmentInfo
+  SegmentInfo
     *s;
 
   SegmentInfo
@@ -168,10 +173,10 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse) 
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((x_offset < 0) || (x_offset >= (ssize_t) image->columns))
     return(MagickFalse);
   if ((y_offset < 0) || (y_offset >= (ssize_t) image->rows))
@@ -180,7 +185,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     return(MagickFalse);
   if (IsGrayColorspace(image->colorspace) != MagickFalse)
     (void) SetImageColorspace(image,sRGBColorspace,exception);
-  if ((image->alpha_trait == UndefinedPixelTrait) &&
+  if (((image->alpha_trait & BlendPixelTrait) == 0) &&
       (draw_info->fill.alpha_trait != UndefinedPixelTrait))
     (void) SetImageAlpha(image,OpaqueAlpha,exception);
   /*
@@ -208,20 +213,20 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   status=MagickTrue;
   start=0;
   s=segment_stack;
-  PushSegmentStack(y_offset,x_offset,x_offset,1);
-  PushSegmentStack(y_offset+1,x_offset,x_offset,-1);
   GetPixelInfo(image,&pixel);
   image_view=AcquireVirtualCacheView(image,exception);
   floodplane_view=AcquireAuthenticCacheView(floodplane_image,exception);
+  PushSegmentStack(y_offset,x_offset,x_offset,1);
+  PushSegmentStack(y_offset+1,x_offset,x_offset,-1);
   while (s > segment_stack)
   {
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     /*
@@ -240,8 +245,8 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       exception);
     if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
       break;
-    p+=x1*GetPixelChannels(image);
-    q+=x1*GetPixelChannels(floodplane_image);
+    p+=(ptrdiff_t) x1*(ssize_t) GetPixelChannels(image);
+    q+=(ptrdiff_t) x1*(ssize_t) GetPixelChannels(floodplane_image);
     for (x=x1; x >= 0; x--)
     {
       if (GetPixelGray(floodplane_image,q) != 0)
@@ -250,7 +255,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       if (IsFuzzyEquivalencePixelInfo(&pixel,target) == invert)
         break;
       SetPixelGray(floodplane_image,QuantumRange,q);
-      p-=GetPixelChannels(image);
+      p-=(ptrdiff_t)GetPixelChannels(image);
       q-=GetPixelChannels(floodplane_image);
     }
     if (SyncCacheViewAuthenticPixels(floodplane_view,exception) == MagickFalse)
@@ -269,10 +274,10 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
         {
           if (x < (ssize_t) image->columns)
             {
-              p=GetCacheViewVirtualPixels(image_view,x,y,image->columns-x,1,
-                exception);
-              q=GetCacheViewAuthenticPixels(floodplane_view,x,y,image->columns-
-                x,1,exception);
+              p=GetCacheViewVirtualPixels(image_view,x,y,(size_t)
+                ((ssize_t) image->columns-x),1,exception);
+              q=GetCacheViewAuthenticPixels(floodplane_view,x,y,(size_t)
+                ((ssize_t) image->columns-x),1,exception);
               if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                 break;
               for ( ; x < (ssize_t) image->columns; x++)
@@ -283,8 +288,8 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
                 if (IsFuzzyEquivalencePixelInfo(&pixel,target) == invert)
                   break;
                 SetPixelGray(floodplane_image,QuantumRange,q);
-                p+=GetPixelChannels(image);
-                q+=GetPixelChannels(floodplane_image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(floodplane_image);
               }
               status=SyncCacheViewAuthenticPixels(floodplane_view,exception);
               if (status == MagickFalse)
@@ -311,23 +316,27 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
             GetPixelInfoPixel(image,p,&pixel);
             if (IsFuzzyEquivalencePixelInfo(&pixel,target) != invert)
               break;
-            p+=GetPixelChannels(image);
-            q+=GetPixelChannels(floodplane_image);
+            p+=(ptrdiff_t) GetPixelChannels(image);
+            q+=(ptrdiff_t) GetPixelChannels(floodplane_image);
           }
         }
       start=x;
     } while (x <= x2);
   }
   status=MagickTrue;
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(floodplane_image,image,image->rows,2)
+#endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     /*
@@ -346,11 +355,24 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     {
       if (GetPixelGray(floodplane_image,p) != 0)
         {
+          PixelInfo
+            fill_color;
+
           GetFillColor(draw_info,x,y,&fill_color,exception);
-          SetPixelViaPixelInfo(image,&fill_color,q);
+          if ((image->channel_mask & RedChannel) != 0)
+            SetPixelRed(image,(Quantum) fill_color.red,q);
+          if ((image->channel_mask & GreenChannel) != 0)
+            SetPixelGreen(image,(Quantum) fill_color.green,q);
+          if ((image->channel_mask & BlueChannel) != 0)
+            SetPixelBlue(image,(Quantum) fill_color.blue,q);
+          if ((image->channel_mask & BlackChannel) != 0)
+            SetPixelBlack(image,(Quantum) fill_color.black,q);
+          if (((image->channel_mask & AlphaChannel) != 0) &&
+              ((image->alpha_trait & BlendPixelTrait) != 0))
+            SetPixelAlpha(image,(Quantum) fill_color.alpha,q);
         }
-      p+=GetPixelChannels(floodplane_image);
-      q+=GetPixelChannels(image);
+      p+=(ptrdiff_t) GetPixelChannels(floodplane_image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -377,7 +399,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 %  vector from one color to another.
 %
 %  Note, the interface of this method will change in the future to support
-%  more than one transistion.
+%  more than one transition.
 %
 %  The format of the GradientImage method is:
 %
@@ -391,7 +413,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 %
 %    o type: the gradient type: linear or radial.
 %
-%    o spread: the gradient spread meathod: pad, reflect, or repeat.
+%    o spread: the gradient spread method: pad, reflect, or repeat.
 %
 %    o start_color: the start color.
 %
@@ -421,10 +443,10 @@ MagickExport MagickBooleanType GradientImage(Image *image,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(stops != (const StopInfo *) NULL);
   assert(number_stops > 0);
+  if (IsEventLogging() != MagickFalse) 
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   draw_info=AcquireDrawInfo();
   gradient=(&draw_info->gradient);
   gradient->type=type;
@@ -498,7 +520,7 @@ MagickExport MagickBooleanType GradientImage(Image *image,
           gradient->gradient_vector.x1=0.0;
           gradient->gradient_vector.y1=0.0;
           gradient->gradient_vector.x2=0.0;
-          gradient->gradient_vector.y2=(double) image->columns-1;
+          gradient->gradient_vector.y2=(double) image->rows-1;
           break;
         }
         case SouthEastGravity:
@@ -645,9 +667,9 @@ MagickExport MagickBooleanType GradientImage(Image *image,
 %
 */
 
-static size_t **DestroyHistogramThreadSet(size_t **histogram)
+static size_t **DestroyHistogramTLS(size_t **histogram)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(histogram != (size_t **) NULL);
@@ -658,9 +680,9 @@ static size_t **DestroyHistogramThreadSet(size_t **histogram)
   return(histogram);
 }
 
-static size_t **AcquireHistogramThreadSet(const size_t count)
+static size_t **AcquireHistogramTLS(const size_t count)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -676,7 +698,7 @@ static size_t **AcquireHistogramThreadSet(const size_t count)
   {
     histogram[i]=(size_t *) AcquireQuantumMemory(count,sizeof(**histogram));
     if (histogram[i] == (size_t *) NULL)
-      return(DestroyHistogramThreadSet(histogram));
+      return(DestroyHistogramTLS(histogram));
   }
   return(histogram);
 }
@@ -714,10 +736,10 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse) 
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   width=GetOptimalKernelWidth2D(radius,sigma);
   linear_image=CloneImage(image,0,0,MagickTrue,exception);
   paint_image=CloneImage(image,0,0,MagickTrue,exception);
@@ -735,7 +757,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       paint_image=DestroyImage(paint_image);
       return((Image *) NULL);
     }
-  histograms=AcquireHistogramThreadSet(NumberPaintBins);
+  histograms=AcquireHistogramTLS(NumberPaintBins);
   if (histograms == (size_t **) NULL)
     {
       linear_image=DestroyImage(linear_image);
@@ -747,8 +769,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   status=MagickTrue;
   progress=0;
-  center=(ssize_t) GetPixelChannels(linear_image)*(linear_image->columns+width)*
-    (width/2L)+GetPixelChannels(linear_image)*(width/2L);
+  center=(ssize_t) (GetPixelChannels(linear_image)*(linear_image->columns+
+    width)*(width/2L)+GetPixelChannels(linear_image)*(width/2L));
   image_view=AcquireVirtualCacheView(linear_image,exception);
   paint_view=AcquireAuthenticCacheView(paint_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -757,16 +779,16 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
 #endif
   for (y=0; y < (ssize_t) linear_image->rows; y++)
   {
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register size_t
+    size_t
       *histogram;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -783,7 +805,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
     histogram=histograms[GetOpenMPThreadId()];
     for (x=0; x < (ssize_t) linear_image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i,
         u;
 
@@ -808,7 +830,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
         for (u=0; u < (ssize_t) width; u++)
         {
           n=(ssize_t) ScaleQuantumToChar(ClampToQuantum(GetPixelIntensity(
-            linear_image,p+GetPixelChannels(linear_image)*(u+k))));
+            linear_image,p+(ssize_t) GetPixelChannels(linear_image)*(u+k))));
           histogram[n]++;
           if (histogram[n] > count)
             {
@@ -831,11 +853,11 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
             SetPixelChannel(paint_image,channel,p[center+i],q);
             continue;
           }
-        SetPixelChannel(paint_image,channel,p[j*GetPixelChannels(linear_image)+
-          i],q);
+        SetPixelChannel(paint_image,channel,p[j*(ssize_t)
+          GetPixelChannels(linear_image)+i],q);
       }
-      p+=GetPixelChannels(linear_image);
-      q+=GetPixelChannels(paint_image);
+      p+=(ptrdiff_t) GetPixelChannels(linear_image);
+      q+=(ptrdiff_t) GetPixelChannels(paint_image);
     }
     if (SyncCacheViewAuthenticPixels(paint_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -856,7 +878,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   }
   paint_view=DestroyCacheView(paint_view);
   image_view=DestroyCacheView(image_view);
-  histograms=DestroyHistogramThreadSet(histograms);
+  histograms=DestroyHistogramTLS(histograms);
   linear_image=DestroyImage(linear_image);
   if (status == MagickFalse)
     paint_image=DestroyImage(paint_image);
@@ -929,7 +951,7 @@ MagickExport MagickBooleanType OpaquePaintImage(Image *image,
   assert(image->signature == MagickCoreSignature);
   assert(target != (PixelInfo *) NULL);
   assert(fill != (PixelInfo *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse) 
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass,exception) == MagickFalse)
     return(MagickFalse);
@@ -951,10 +973,10 @@ MagickExport MagickBooleanType OpaquePaintImage(Image *image,
     PixelInfo
       pixel;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -990,7 +1012,7 @@ MagickExport MagickBooleanType OpaquePaintImage(Image *image,
           if ((traits & UpdatePixelTrait) != 0)
             SetPixelAlpha(image,(Quantum) conform_fill.alpha,q);
         }
-      q+=GetPixelChannels(image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -1076,11 +1098,11 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
   assert(target != (PixelInfo *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse) 
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass,exception) == MagickFalse)
     return(MagickFalse);
-  if (image->alpha_trait == UndefinedPixelTrait)
+  if ((image->alpha_trait & BlendPixelTrait) == 0)
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel,exception);
   /*
     Make image color transparent.
@@ -1098,10 +1120,10 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
     PixelInfo
       pixel;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1118,7 +1140,7 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
       GetPixelInfoPixel(image,q,&pixel);
       if (IsFuzzyEquivalencePixelInfo(&pixel,target) != invert)
         SetPixelAlpha(image,opacity,q);
-      q+=GetPixelChannels(image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -1205,11 +1227,11 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
   assert(image->signature == MagickCoreSignature);
   assert(high != (PixelInfo *) NULL);
   assert(low != (PixelInfo *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse) 
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass,exception) == MagickFalse)
     return(MagickFalse);
-  if (image->alpha_trait == UndefinedPixelTrait)
+  if ((image->alpha_trait & BlendPixelTrait) == 0)
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel,exception);
   /*
     Make image color transparent.
@@ -1229,10 +1251,10 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
     PixelInfo
       pixel;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -1253,7 +1275,7 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
         MagickFalse;
       if (match != invert)
         SetPixelAlpha(image,opacity,q);
-      q+=GetPixelChannels(image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
